"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator.throw(value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const ConnectionIsNotSetError_1 = require("./error/ConnectionIsNotSetError");
const BaseDriver_1 = require("./BaseDriver");
const PostgresSchemaBuilder_1 = require("../schema-builder/PostgresSchemaBuilder");
/**
 * This driver organizes work with postgres database.
 */
class PostgresDriver extends BaseDriver_1.BaseDriver {
    // -------------------------------------------------------------------------
    // Constructor
    // -------------------------------------------------------------------------
    constructor(connectionOptions, postgres) {
        super();
        this.connectionOptions = connectionOptions;
        // -------------------------------------------------------------------------
        // Public Properties
        // -------------------------------------------------------------------------
        this.isResultsLowercase = true;
        // if driver dependency is not given explicitly, then try to load it via "require"
        if (!postgres && require) {
            try {
                postgres = require("pg");
            }
            catch (e) {
                throw new Error("Postgres package has not been found installed. Try to install it: npm install pg --save");
            }
        }
        else {
            throw new Error("Cannot load postgres driver dependencies. Try to install all required dependencies.");
        }
        this.postgres = postgres;
    }
    // -------------------------------------------------------------------------
    // Getter Methods
    // -------------------------------------------------------------------------
    /**
     * Access to the native implementation of the database.
     */
    get native() {
        return this.postgres;
    }
    /**
     * Access to the native connection to the database.
     */
    get nativeConnection() {
        return this.postgresConnection;
    }
    /**
     * Database name to which this connection is made.
     */
    get db() {
        // if (this.postgresConnection && this.postgresConnection.config.database)
        //     return this.postgresConnection.config.database;
        if (this.connectionOptions.database)
            return this.connectionOptions.database;
        throw new Error("Cannot get the database name. Since database name is not explicitly given in configuration " +
            "(maybe connection url is used?), database name cannot be retrieved until connection is made.");
    }
    // -------------------------------------------------------------------------
    // Public Methods
    // -------------------------------------------------------------------------
    retrieveDatabaseConnection() {
        /*if (this.postgresPool) { // todo: rename to databaseConnectionPool
            return new Promise((ok, fail) => {
                this.postgresPool.getConnection((err: any, connection: any) => {
                    if (err) {
                        fail(err);
                        return;
                    }

                    ok(connection);
                });
            });
        }*/
        if (this.postgresConnection)
            return Promise.resolve(this.postgresConnection);
        throw new ConnectionIsNotSetError_1.ConnectionIsNotSetError("mysql");
    }
    releaseDatabaseConnection(dbConnection) {
        // if (this.mysqlPool) {
        //     dbConnection.connection.release();
        // }
        return Promise.resolve();
    }
    /**
     * Creates a schema builder which can be used to build database/table schemas.
     */
    createSchemaBuilder(dbConnection) {
        return new PostgresSchemaBuilder_1.PostgresSchemaBuilder(this, dbConnection);
    }
    /**
     * Performs connection to the database based on given connection options.
     */
    connect() {
        return new Promise((ok, fail) => {
            const options = Object.assign({}, {
                host: this.connectionOptions.host,
                user: this.connectionOptions.username,
                password: this.connectionOptions.password,
                database: this.connectionOptions.database,
                port: this.connectionOptions.port
            }, this.connectionOptions.extra || {});
            this.postgresConnection = {
                id: 1,
                connection: new this.postgres.Client(options),
                isTransactionActive: false
            };
            this.postgresConnection.connection.connect((err) => err ? fail(err) : ok());
        });
    }
    /**
     * Closes connection with database.
     */
    disconnect() {
        this.checkIfConnectionSet();
        return new Promise((ok, fail) => {
            if (this.postgresConnection) {
                this.postgresConnection.connection.end(); // todo: check if it can emit errors
                this.postgresConnection = undefined;
            }
            ok();
        });
    }
    /**
     * Escapes given value.
     */
    escape(dbConnection, value) {
        return value; // TODO: this.postgresConnection.escape(value);
    }
    /**
     * Executes a given SQL query.
     */
    query(dbConnection, query, parameters) {
        this.checkIfConnectionSet();
        // console.log("query: ", query);
        // console.log("parameters: ", parameters);
        this.logQuery(query);
        return new Promise((ok, fail) => {
            if (!this.postgresConnection)
                return fail(new ConnectionIsNotSetError_1.ConnectionIsNotSetError("postgres"));
            this.postgresConnection.connection.query(query, parameters, (err, result) => {
                if (err) {
                    this.logFailedQuery(query);
                    this.logQueryError(err);
                    fail(err);
                }
                else {
                    ok(result.rows);
                }
            });
        });
    }
    /**
     * Clears all tables in the currently connected database.
     */
    clearDatabase(dbConnection) {
        this.checkIfConnectionSet();
        const dropTablesQuery = `SELECT 'DROP TABLE IF EXISTS "' || tablename || '" CASCADE;' as q FROM pg_tables WHERE schemaname = 'public';`;
        return this.query(dbConnection, dropTablesQuery)
            .then(results => Promise.all(results.map(q => this.query(dbConnection, q["q"]))))
            .then(() => { });
    }
    buildParameters(sql, parameters) {
        const builtParameters = [];
        Object.keys(parameters).forEach((key, index) => {
            // const value = this.parameters[key] !== null && this.parameters[key] !== undefined ? this.driver.escape(this.parameters[key]) : "NULL";
            sql = sql.replace(new RegExp(":" + key, "g"), (str) => {
                builtParameters.push(parameters[key]);
                return "";
            }); // todo: make replace only in value statements, otherwise problems
        });
        return builtParameters;
    }
    replaceParameters(sql, parameters) {
        Object.keys(parameters).forEach((key, index) => {
            // const value = parameters[key] !== null && parameters[key] !== undefined ? this.driver.escape(parameters[key]) : "NULL";
            sql = sql.replace(new RegExp(":" + key, "g"), (str) => {
                return "$" + (index + 1);
            }); // todo: make replace only in value statements, otherwise problems
        });
        return sql;
    }
    /**
     * Insert a new row into given table.
     */
    insert(dbConnection, tableName, keyValues, idColumnName) {
        this.checkIfConnectionSet();
        const columns = Object.keys(keyValues).join(",");
        const values = Object.keys(keyValues).map((key, index) => "$" + (index + 1)).join(","); // todo: escape here
        const params = Object.keys(keyValues).map(key => keyValues[key]);
        let query = `INSERT INTO ${tableName}(${columns}) VALUES (${values})`;
        if (idColumnName) {
            query += " RETURNING " + idColumnName;
        }
        return this.query(dbConnection, query, params).then(result => {
            if (idColumnName)
                return result[0][idColumnName];
            return undefined;
        });
    }
    /**
     * Updates rows that match given conditions in the given table.
     */
    update(dbConnection, tableName, valuesMap, conditions) {
        return __awaiter(this, void 0, void 0, function* () {
            this.checkIfConnectionSet();
            const updateValues = this.parametrizeObjectMap(valuesMap).join(",");
            const conditionString = this.parametrizeObjectMap(conditions, Object.keys(valuesMap).length).join(" AND ");
            const updateParams = Object.keys(valuesMap).map(key => valuesMap[key]);
            const conditionParams = Object.keys(conditions).map(key => conditions[key]);
            const query = `UPDATE ${tableName} SET ${updateValues} ${conditionString ? (" WHERE " + conditionString) : ""}`;
            // console.log("executing update: ", query);
            yield this.query(dbConnection, query, updateParams.concat(conditionParams));
        });
    }
    /**
     * Deletes from the given table by a given conditions.
     */
    delete(dbConnection, tableName, conditions) {
        return __awaiter(this, void 0, void 0, function* () {
            this.checkIfConnectionSet();
            const conditionString = this.parametrizeObjectMap(conditions).join(" AND ");
            const params = Object.keys(conditions).map(key => conditions[key]);
            const query = `DELETE FROM ${tableName} WHERE ${conditionString}`;
            yield this.query(dbConnection, query, params);
        });
    }
    // -------------------------------------------------------------------------
    // Protected Methods
    // -------------------------------------------------------------------------
    parametrizeObjectMap(objectMap, startIndex = 0) {
        return Object.keys(objectMap).map((key, index) => {
            const value = objectMap[key];
            // if (value === null || value === undefined) { // todo: I think we dont really need this
            //     return key + "=NULL";
            // } else {
            return key + "=$" + (startIndex + index + 1);
            // }
        });
    }
    checkIfConnectionSet() {
        if (!this.postgresConnection)
            throw new ConnectionIsNotSetError_1.ConnectionIsNotSetError("postgres");
    }
}
exports.PostgresDriver = PostgresDriver;

//# sourceMappingURL=PostgresDriver.js.map
