/// <reference types="chai" />
import { Connection } from "../connection/Connection";
import { QueryBuilder } from "../query-builder/QueryBuilder";
import { Repository } from "../repository/Repository";
import { ObjectType } from "../common/ObjectType";
import { ReactiveRepository } from "../repository/ReactiveRepository";
import { TreeRepository } from "../repository/TreeRepository";
import { ObjectLiteral } from "../common/ObjectLiteral";
import { TreeReactiveRepository } from "../repository/TreeReactiveRepository";
/**
 * Common functions shared between different manager types.
 */
export declare abstract class BaseEntityManager {
    protected connection: Connection;
    constructor(connection: Connection);
    /**
     * Gets repository for the given entity class.
     */
    getRepository<Entity>(entityClass: ObjectType<Entity>): Repository<Entity>;
    /**
     * Gets repository for the given entity name.
     */
    getRepository<Entity>(entityName: string): Repository<Entity>;
    /**
     * Gets tree repository for the given entity class.
     */
    getTreeRepository<Entity>(entityClass: ObjectType<Entity>): TreeRepository<Entity>;
    /**
     * Gets tree repository for the given entity name.
     */
    getTreeRepository<Entity>(entityName: string): TreeRepository<Entity>;
    /**
     * Gets reactive repository for the given entity class.
     */
    getReactiveRepository<Entity>(entityClass: ObjectType<Entity>): ReactiveRepository<Entity>;
    /**
     * Gets reactive repository for the given entity name.
     */
    getReactiveRepository<Entity>(entityName: string): ReactiveRepository<Entity>;
    /**
     * Gets reactive tree repository for the given entity class.
     */
    getReactiveTreeRepository<Entity>(entityClass: ObjectType<Entity>): TreeReactiveRepository<Entity>;
    /**
     * Gets reactive tree repository for the given entity name.
     */
    getReactiveTreeRepository<Entity>(entityName: string): TreeReactiveRepository<Entity>;
    /**
     * Checks if entity has an id.
     */
    hasId(entity: Object): boolean;
    /**
     * Checks if entity of given schema name has an id.
     */
    hasId(target: string, entity: Object): boolean;
    /**
     * Creates a new query builder that can be used to build an sql query.
     */
    createQueryBuilder<Entity>(entityClass: ObjectType<Entity>, alias: string): QueryBuilder<Entity>;
    /**
     * Creates a new entity instance.
     */
    create<Entity>(entityClass: ObjectType<Entity>): Entity;
    /**
     * Creates a new entity instance and copies all entity properties from this object into a new entity.
     * Note that it copies only properties that present in entity schema.
     */
    create<Entity>(entityClass: ObjectType<Entity>, plainObject: Object): Entity;
    /**
     * Creates a new entities and copies all entity properties from given objects into their new entities.
     * Note that it copies only properties that present in entity schema.
     */
    create<Entity>(entityClass: ObjectType<Entity>, plainObjects: Object[]): Entity[];
    /**
     * Creates a new entity from the given plan javascript object. If entity already exist in the database, then
     * it loads it (and everything related to it), replaces all values with the new ones from the given object
     * and returns this new entity. This new entity is actually a loaded from the db entity with all properties
     * replaced from the new object.
     */
    preload<Entity>(entityClass: ObjectType<Entity>, object: Object): Promise<Entity>;
    /**
     * Merges two entities into one new entity.
     */
    merge<Entity>(entityClass: ObjectType<Entity>, ...objects: ObjectLiteral[]): Entity;
    /**
     * We are using a private function to avoid type problems, because we are sending a Function everywhere in the
     * code, and obtainRepository does not accept a Function, and only ConstructorFunction it can accept - it brings
     * us type problems. To avoid this we are using private function here.
     */
    protected obtainRepository<Entity>(entityClassOrName: ObjectType<Entity> | string): Repository<Entity>;
    /**
     * We are using a private function to avoid type problems, because we are sending a Function everywhere in the
     * code, and obtainTreeRepository does not accept a Function, and only ConstructorFunction it can accept - it brings
     * us type problems. To avoid this we are using private function here.
     */
    protected obtainTreeRepository<Entity>(entityClassOrName: ObjectType<Entity> | string): TreeRepository<Entity>;
    /**
     * We are using a private function to avoid type problems, because we are sending a Function everywhere in the
     * code, and obtainReactiveRepository does not accept a Function, and only ConstructorFunction it can accept - it brings
     * us type problems. To avoid this we are using private function here.
     */
    protected obtainReactiveRepository<Entity>(entityClassOrName: ObjectType<Entity> | string): ReactiveRepository<Entity>;
    /**
     * We are using a private function to avoid type problems, because we are sending a Function everywhere in the
     * code, and obtainReactiveTreeRepository does not accept a Function, and only ConstructorFunction it can accept - it brings
     * us type problems. To avoid this we are using private function here.
     */
    protected obtainReactiveTreeRepository<Entity>(entityClassOrName: ObjectType<Entity> | string): TreeReactiveRepository<Entity>;
}
