/// <reference types="chai" />
import { QueryBuilder } from "../query-builder/QueryBuilder";
import { FindOptions } from "./FindOptions";
import { Repository } from "./Repository";
import * as Rx from "rxjs/Rx";
import { ObjectLiteral } from "../common/ObjectLiteral";
/**
 * Repository is supposed to work with your entity objects. Find entities, insert, update, delete, etc.
 * This version of Repository is using rxjs library and Observables instead of promises.
 *
 * @experimental
 */
export declare class ReactiveRepository<Entity> {
    protected repository: Repository<Entity>;
    constructor(repository: Repository<Entity>);
    /**
     * Repository's target is an object (or unique string in the case if entity is loaded from a schema) that is managed
     * by this repository.
     */
    readonly target: Function | string;
    /**
     * Checks if entity has an id.
     */
    hasId(entity: Entity): boolean;
    /**
     * Creates a new query builder that can be used to build a sql query.
     */
    createQueryBuilder(alias: string): QueryBuilder<Entity>;
    /**
     * Creates a new entity instance.
     */
    create(): Entity;
    /**
     * Creates a new entities and copies all entity properties from given objects into their new entities.
     * Note that it copies only properties that present in entity schema.
     */
    create(plainObjects: Object[]): Entity[];
    /**
     * Creates a new entity instance and copies all entity properties from this object into a new entity.
     * Note that it copies only properties that present in entity schema.
     */
    create(plainObject: Object): Entity;
    /**
     * Creates a new entity from the given plan javascript object. If entity already exist in the database, then
     * it loads it (and everything related to it), replaces all values with the new ones from the given object
     * and returns this new entity. This new entity is actually a loaded from the db entity with all properties
     * replaced from the new object.
     */
    preload(object: Object): Rx.Observable<Entity>;
    /**
     * Merges two entities into one new entity.
     */
    merge(...objects: ObjectLiteral[]): Entity;
    /**
     * Persists (saves) a given entity in the database. If entity does not exist in the database then it inserts it,
     * else if entity already exist in the database then it updates it.
     */
    persist(entity: Entity): Rx.Observable<Entity>;
    /**
     * Removes a given entity from the database.
     */
    remove(entity: Entity): Rx.Observable<Entity>;
    /**
     * Finds all entities.
     */
    find(): Rx.Observable<Entity[]>;
    /**
     * Finds entities that match given conditions.
     */
    find(conditions: Object): Rx.Observable<Entity[]>;
    /**
     * Finds entities that match given find options.
     */
    find(options: FindOptions): Rx.Observable<Entity[]>;
    /**
     * Finds entities that match given conditions.
     */
    find(conditions: Object, options: FindOptions): Rx.Observable<Entity[]>;
    /**
     * Finds entities that match given conditions.
     */
    findAndCount(): Rx.Observable<[Entity[], number]>;
    /**
     * Finds entities that match given conditions.
     */
    findAndCount(conditions: Object): Rx.Observable<[Entity[], number]>;
    /**
     * Finds entities that match given conditions.
     */
    findAndCount(options: FindOptions): Rx.Observable<[Entity[], number]>;
    /**
     * Finds entities that match given conditions.
     */
    findAndCount(conditions: Object, options: FindOptions): Rx.Observable<[Entity[], number]>;
    /**
     * Finds first entity that matches given conditions.
     */
    findOne(): Rx.Observable<Entity>;
    /**
     * Finds first entity that matches given conditions.
     */
    findOne(conditions: Object): Rx.Observable<Entity>;
    /**
     * Finds first entity that matches given conditions.
     */
    findOne(options: FindOptions): Rx.Observable<Entity>;
    /**
     * Finds first entity that matches given conditions.
     */
    findOne(conditions: Object, options: FindOptions): Rx.Observable<Entity>;
    /**
     * Finds entity with given id.
     */
    findOneById(id: any, options?: FindOptions): Rx.Observable<Entity>;
    /**
     * Executes raw SQL query and returns raw database results.
     */
    query(query: string): Rx.Observable<any>;
    /**
     * Wraps given function execution (and all operations made there) in a transaction.
     */
    transaction(runInTransaction: () => Promise<any>): Rx.Observable<any>;
}
