"use strict";
const EventListenerTypes_1 = require("../metadata/types/EventListenerTypes");
/**
 * Broadcaster provides a helper methods to broadcast events to the subscribers.
 */
class Broadcaster {
    // -------------------------------------------------------------------------
    // Constructor
    // -------------------------------------------------------------------------
    constructor(entityMetadatas, subscriberMetadatas, entityListeners) {
        this.entityMetadatas = entityMetadatas;
        this.subscriberMetadatas = subscriberMetadatas;
        this.entityListeners = entityListeners;
    }
    // -------------------------------------------------------------------------
    // Accessors
    // -------------------------------------------------------------------------
    broadcastBeforeInsertEvent(entity) {
        const subscribers = this.subscriberMetadatas
            .filter(subscriber => !!subscriber)
            .map(subscriber => {
            if (subscriber.beforeInsert)
                return subscriber.beforeInsert({ entity: entity });
        });
        const listeners = this.entityListeners
            .filter(listener => listener.type === EventListenerTypes_1.EventListenerTypes.BEFORE_INSERT)
            .filter(listener => listener.target === entity.constructor)
            .map(entityListener => entity[entityListener.propertyName]());
        return Promise.all(subscribers.concat(listeners)).then(() => { });
    }
    broadcastBeforeUpdateEvent(entity, updatedColumns) {
        const subscribers = this.subscriberMetadatas
            .filter(subscriber => this.isAllowedSubscribers(subscriber, entity))
            .map(subscriber => {
            if (!subscriber.beforeUpdate)
                return undefined;
            return subscriber.beforeUpdate({ entity: entity, updatedColumns: updatedColumns });
        })
            .filter(subscriber => !!subscriber);
        const listeners = this.entityListeners
            .filter(listener => listener.type === EventListenerTypes_1.EventListenerTypes.BEFORE_UPDATE)
            .filter(listener => listener.target === entity.constructor)
            .map(entityListener => entity[entityListener.propertyName]());
        return Promise.all(subscribers.concat(listeners)).then(() => { });
    }
    broadcastBeforeRemoveEvent(entity, entityId) {
        const subscribers = this.subscriberMetadatas
            .filter(subscriber => this.isAllowedSubscribers(subscriber, entity))
            .map(subscriber => {
            if (!subscriber.beforeRemove)
                return undefined;
            return subscriber.beforeRemove({ entity: entity, entityId: entityId });
        })
            .filter(subscriber => !!subscriber);
        const listeners = this.entityListeners
            .filter(listener => listener.type === EventListenerTypes_1.EventListenerTypes.BEFORE_REMOVE)
            .filter(listener => listener.target === entity.constructor)
            .map(entityListener => entity[entityListener.propertyName]());
        return Promise.all(subscribers.concat(listeners)).then(() => { });
    }
    broadcastAfterInsertEvent(entity) {
        const subscribers = this.subscriberMetadatas
            .filter(subscriber => this.isAllowedSubscribers(subscriber, entity))
            .map(subscriber => {
            if (!subscriber.afterInsert)
                return undefined;
            return subscriber.afterInsert({ entity: entity });
        })
            .filter(subscriber => !!subscriber);
        const listeners = this.entityListeners
            .filter(listener => listener.type === EventListenerTypes_1.EventListenerTypes.AFTER_INSERT)
            .filter(listener => listener.target === entity.constructor)
            .map(entityListener => entity[entityListener.propertyName]());
        return Promise.all(subscribers.concat(listeners)).then(() => { });
    }
    broadcastAfterUpdateEvent(entity, updatedColumns) {
        const subscribers = this.subscriberMetadatas
            .filter(subscriber => this.isAllowedSubscribers(subscriber, entity))
            .map(subscriber => {
            if (!subscriber.afterUpdate)
                return undefined;
            return subscriber.afterUpdate({ entity: entity, updatedColumns: updatedColumns });
        })
            .filter(subscriber => !!subscriber);
        const listeners = this.entityListeners
            .filter(listener => listener.type === EventListenerTypes_1.EventListenerTypes.AFTER_UPDATE)
            .filter(listener => listener.target === entity.constructor)
            .map(entityListener => entity[entityListener.propertyName]());
        return Promise.all(subscribers.concat(listeners)).then(() => { });
    }
    broadcastAfterRemoveEvent(entity, entityId) {
        const subscribers = this.subscriberMetadatas
            .filter(subscriber => this.isAllowedSubscribers(subscriber, entity))
            .map(subscriber => {
            if (!subscriber.afterRemove)
                return undefined;
            return subscriber.afterRemove({ entity: entity, entityId: entityId });
        })
            .filter(subscriber => !!subscriber);
        const listeners = this.entityListeners
            .filter(listener => listener.type === EventListenerTypes_1.EventListenerTypes.AFTER_REMOVE)
            .filter(listener => listener.target === entity.constructor)
            .map(entityListener => entity[entityListener.propertyName]());
        return Promise.all(subscribers.concat(listeners)).then(() => { });
    }
    broadcastLoadEvents(target, entity) {
        if (entity instanceof Promise)
            return Promise.resolve();
        const metadata = this.entityMetadatas.findByTarget(target);
        let promises = [];
        metadata
            .relations
            .filter(relation => entity.hasOwnProperty(relation.propertyName))
            .map(relation => {
            const value = this.getEntityRelationValue(relation, entity);
            if (value instanceof Array) {
                promises = promises.concat(this.broadcastLoadEventsForAll(relation.inverseEntityMetadata.target, value));
            }
            else if (value) {
                promises.push(this.broadcastLoadEvents(relation.inverseEntityMetadata.target, value));
            }
        });
        this.subscriberMetadatas
            .filter(subscriber => this.isAllowedSubscribers(subscriber, entity))
            .forEach(subscriber => {
            if (subscriber.afterLoad)
                promises.push(subscriber.afterLoad(entity));
        });
        this.entityListeners
            .filter(listener => listener.type === EventListenerTypes_1.EventListenerTypes.AFTER_LOAD)
            .filter(listener => listener.target === entity.constructor)
            .forEach(listener => promises.push(entity[listener.propertyName]()));
        return Promise.all(promises).then(() => { });
    }
    broadcastLoadEventsForAll(target, entities) {
        const promises = entities.map(entity => this.broadcastLoadEvents(target, entity));
        return Promise.all(promises).then(() => { });
    }
    // -------------------------------------------------------------------------
    // Private Methods
    // -------------------------------------------------------------------------
    getEntityRelationValue(relation, entity) {
        return relation.isLazy ? entity["__" + relation.propertyName + "__"] : entity[relation.propertyName];
    }
    isAllowedSubscribers(subscriber, cls) {
        return !subscriber.listenTo ||
            !subscriber.listenTo() ||
            subscriber.listenTo() === Object ||
            subscriber.listenTo() === cls;
    }
}
exports.Broadcaster = Broadcaster;

//# sourceMappingURL=Broadcaster.js.map
